@@ -28,17 +28,14 @@ def pytest_plugin_registered(plugin, manager):
 
 @hookspec(historic=True)
 def pytest_addoption(parser):
-    """register argparse-style options and ini-style config values.
+    """register argparse-style options and ini-style config values,
+    called once at the beginning of a test run.
 
-    .. warning::
+    .. note::
 
-        This function must be implemented in a :ref:`plugin <pluginorder>`
-        and is called once at the beginning of a test run.
-
-        Implementing this hook from ``conftest.py`` files is **strongly**
-        discouraged because ``conftest.py`` files are lazily loaded and
-        may give strange *unknown option* errors depending on the directory
-        ``py.test`` is invoked from.
+        This function should be implemented only in plugins or ``conftest.py``
+        files situated at the tests root directory due to how pytest
+        :ref:`discovers plugins during startup <pluginorder>`.
 
     :arg parser: To add command line options, call
         :py:func:`parser.addoption(...) <_pytest.config.Parser.addoption>`.
@@ -84,7 +81,7 @@ def pytest_cmdline_main(config):
     """ called for performing the main command line action. The default
     implementation will invoke the configure hooks and runtest_mainloop. """
 
-def pytest_load_initial_conftests(args, early_config, parser):
+def pytest_load_initial_conftests(early_config, parser, args):
     """ implements the loading of initial conftest files ahead
     of command line option parsing. """
 
@@ -159,6 +156,12 @@ def pytest_pyfunc_call(pyfuncitem):
 def pytest_generate_tests(metafunc):
     """ generate (multiple) parametrized calls to a test function."""
 
+@hookspec(firstresult=True)
+def pytest_make_parametrize_id(config, val):
+    """Return a user-friendly string representation of the given ``val`` that will be used
+    by @pytest.mark.parametrize calls. Return None if the hook doesn't know about ``val``.
+    """
+
 # -------------------------------------------------------------------------
 # generic runtest related hooks
 # -------------------------------------------------------------------------
@@ -215,6 +218,19 @@ def pytest_runtest_logreport(report):
     """ process a test setup/call/teardown report relating to
     the respective phase of executing a test. """
 
+# -------------------------------------------------------------------------
+# Fixture related hooks
+# -------------------------------------------------------------------------
+
+@hookspec(firstresult=True)
+def pytest_fixture_setup(fixturedef, request):
+    """ performs fixture setup execution. """
+
+def pytest_fixture_post_finalizer(fixturedef):
+    """ called after fixture teardown, but before the cache is cleared so
+    the fixture result cache ``fixturedef.cached_result`` can
+    still be accessed."""
+
 # -------------------------------------------------------------------------
 # test session related hooks
 # -------------------------------------------------------------------------
@@ -253,7 +269,7 @@ def pytest_report_header(config, startdir):
 def pytest_report_teststatus(report):
     """ return result-category, shortletter and verbose word for reporting."""
 
-def pytest_terminal_summary(terminalreporter):
+def pytest_terminal_summary(terminalreporter, exitstatus):
     """ add additional section in terminal summary reporting.  """
 
 
@@ -282,15 +298,17 @@ def pytest_keyboard_interrupt(excinfo):
     """ called for keyboard interrupt. """
 
 def pytest_exception_interact(node, call, report):
-    """ (experimental, new in 2.4) called when
-    an exception was raised which can potentially be
+    """called when an exception was raised which can potentially be
     interactively handled.
 
     This hook is only called if an exception was raised
-    that is not an internal exception like "skip.Exception".
+    that is not an internal exception like ``skip.Exception``.
     """
 
-def pytest_enter_pdb():
+def pytest_enter_pdb(config):
     """ called upon pdb.set_trace(), can be used by plugins to take special
     action just before the python debugger enters in interactive mode.
+
+    :arg config: pytest config object
+    :type config: _pytest.config.Config
     """
