@@ -118,13 +118,10 @@ class _NodeReporter(object):
 
     def _write_captured_output(self, report):
         for capname in ('out', 'err'):
-            allcontent = ""
-            for name, content in report.get_sections("Captured std%s" %
-                                                     capname):
-                allcontent += content
-            if allcontent:
+            content = getattr(report, 'capstd' + capname)
+            if content:
                 tag = getattr(Junit, 'system-' + capname)
-                self.append(tag(bin_xml_escape(allcontent)))
+                self.append(tag(bin_xml_escape(content)))
 
     def append_pass(self, report):
         self.add_stats('passed')
@@ -159,8 +156,12 @@ class _NodeReporter(object):
             Junit.skipped, "collection skipped", report.longrepr)
 
     def append_error(self, report):
+        if getattr(report, 'when', None) == 'teardown':
+            msg = "test teardown failure"
+        else:
+            msg = "test setup failure"
         self._add_simple(
-            Junit.error, "test setup failure", report.longrepr)
+            Junit.error, msg, report.longrepr)
         self._write_captured_output(report)
 
     def append_skipped(self, report):
@@ -186,8 +187,8 @@ class _NodeReporter(object):
 
 @pytest.fixture
 def record_xml_property(request):
-    """Fixture that adds extra xml properties to the tag for the calling test.
-    The fixture is callable with (name, value), with value being automatically
+    """Add extra xml properties to the tag for the calling test.
+    The fixture is callable with ``(name, value)``, with value being automatically
     xml-encoded.
     """
     request.node.warn(
@@ -265,6 +266,7 @@ class LogXML(object):
         ], 0)
         self.node_reporters = {}  # nodeid -> _NodeReporter
         self.node_reporters_ordered = []
+        self.global_properties = []
 
     def finalize(self, report):
         nodeid = getattr(report, 'nodeid', report)
@@ -284,9 +286,12 @@ class LogXML(object):
         if key in self.node_reporters:
             # TODO: breasks for --dist=each
             return self.node_reporters[key]
+
         reporter = _NodeReporter(nodeid, self)
+
         self.node_reporters[key] = reporter
         self.node_reporters_ordered.append(reporter)
+
         return reporter
 
     def add_stats(self, key):
@@ -369,10 +374,12 @@ class LogXML(object):
         suite_stop_time = time.time()
         suite_time_delta = suite_stop_time - self.suite_start_time
 
-        numtests = self.stats['passed'] + self.stats['failure'] + self.stats['skipped']
+        numtests = self.stats['passed'] + self.stats['failure'] + self.stats['skipped'] + self.stats['error']
 
         logfile.write('<?xml version="1.0" encoding="utf-8"?>')
+
         logfile.write(Junit.testsuite(
+            self._get_global_properties_node(),
             [x.to_xml() for x in self.node_reporters_ordered],
             name="pytest",
             errors=self.stats['error'],
@@ -385,3 +392,18 @@ class LogXML(object):
     def pytest_terminal_summary(self, terminalreporter):
         terminalreporter.write_sep("-",
                                    "generated xml file: %s" % (self.logfile))
+
+    def add_global_property(self, name, value):
+        self.global_properties.append((str(name), bin_xml_escape(value)))
+
+    def _get_global_properties_node(self):
+        """Return a Junit node containing custom properties, if any.
+        """
+        if self.global_properties:
+            return Junit.properties(
+                    [
+                        Junit.property(name=name, value=value)
+                        for name, value in self.global_properties
+                    ]
+            )
+        return ''
