@@ -1,13 +1,12 @@
 """ discovery and running of std-library "unittest" style tests. """
 from __future__ import absolute_import
-import traceback
+
 import sys
+import traceback
 
 import pytest
-import py
-
-
 # for transfering markers
+import _pytest._code
 from _pytest.python import transfer_markers
 from _pytest.skipping import MarkEvaluator
 
@@ -24,9 +23,10 @@ def pytest_pycollect_makeitem(collector, name, obj):
 
 
 class UnitTestCase(pytest.Class):
-    nofuncargs = True  # marker for fixturemanger.getfixtureinfo()
-                       # to declare that our children do not support funcargs
-                       #
+    # marker for fixturemanger.getfixtureinfo()
+    # to declare that our children do not support funcargs
+    nofuncargs = True
+                                              
     def setup(self):
         cls = self.obj
         if getattr(cls, '__unittest_skip__', False):
@@ -50,6 +50,8 @@ class UnitTestCase(pytest.Class):
         foundsomething = False
         for name in loader.getTestCaseNames(self.obj):
             x = getattr(self.obj, name)
+            if not getattr(x, '__test__', True):
+                continue
             funcobj = getattr(x, 'im_func', x)
             transfer_markers(funcobj, cls, module)
             yield TestCaseFunction(name, parent=self)
@@ -92,6 +94,9 @@ class TestCaseFunction(pytest.Function):
     def teardown(self):
         if hasattr(self._testcase, 'teardown_method'):
             self._testcase.teardown_method(self._obj)
+        # Allow garbage collection on TestCase instance attributes.
+        self._testcase = None
+        self._obj = None
 
     def startTest(self, testcase):
         pass
@@ -100,7 +105,7 @@ class TestCaseFunction(pytest.Function):
         # unwrap potential exception info (see twisted trial support below)
         rawexcinfo = getattr(rawexcinfo, '_rawexcinfo', rawexcinfo)
         try:
-            excinfo = py.code.ExceptionInfo(rawexcinfo)
+            excinfo = _pytest._code.ExceptionInfo(rawexcinfo)
         except TypeError:
             try:
                 try:
@@ -116,7 +121,7 @@ class TestCaseFunction(pytest.Function):
             except KeyboardInterrupt:
                 raise
             except pytest.fail.Exception:
-                excinfo = py.code.ExceptionInfo()
+                excinfo = _pytest._code.ExceptionInfo()
         self.__dict__.setdefault('_excinfo', []).append(excinfo)
 
     def addError(self, testcase, rawexcinfo):
@@ -148,7 +153,12 @@ class TestCaseFunction(pytest.Function):
         pass
 
     def runtest(self):
-        self._testcase(result=self)
+        if self.config.pluginmanager.get_plugin("pdbinvoke") is None:
+            self._testcase(result=self)
+        else:
+            # disables tearDown and cleanups for post mortem debugging (see #1890)
+            self._testcase.debug()
+
 
     def _prunetraceback(self, excinfo):
         pytest.Function._prunetraceback(self, excinfo)
